# A simple class to create and manage locally maintained repositories using
# tar file containing the RPMs.
#
# @summary Creates and hosts the specified RPM repositories.
#
# @param mirror_fqdn [String]
#   Fully Qualified Domain Name which will be used as the apache vhost name on
#   this server. Note that this FQDN should resolve to this server.
#
# @param source_base_url [String]
#   This specifies the URL from which the repo tar should be extracted. Note
#   that the application assumes all of the tar files are located within the
#   specified directory. For example, `http://bin.arden/data` should contain a
#   file for each repo of the form `${source_prefix}-${version}.tar.`
#
# @param repo_base_path [String]
#   Absolute path to the parent directory of the yum repository
#       
# @param repo_base_cache_path [String]
#   Absolute path to the parent directory of the yum repository cache
#   directory.
#
# @param repo_list [Hash]
#   A hash where the keys are the names of the repos to be deployed. The
#   following options are supported for each repo:
#   * version       [String]  - version of the source repo file
#   * source_prefix [String]  - prefix of the file, defaults to the repo name.
#
#   @note At least one repository must be specified
#
# @example Definition of three repositories
#   class { 'local_rpm_repo':
#     source_base_url => 'http://bin.example.com/data/',
#     mirror_fqdn     => 'yum.example.com',
#     repo_list       => {
#       'repo1' => { version => '1.0.0.0' },
#       'repo2' => { version => '1.0.0.0' },
#       'repo3' => { version => '1.0.0.0' },
#     },
#   }
class local_rpm_repo (
  String  $mirror_fqdn = undef,
  String  $source_base_url = undef,
  String  $repo_base_path = undef,
  String  $repo_base_cache_path = undef,
  Hash[String, Optional[Hash[String, String]]] $repo_list = [],
) {
  # Ensure Apache is installed
  include apache

  # Fail if no repo is specified
  if empty($repo_list) {
    fail('At least one repo must be specified!')
  }

  # Ensure both the mirror and source_base_url's are specified
  if $mirror_fqdn == undef {
    fail('The mirror FQDN must be specified!')
  } elsif $source_base_url == undef {
    fail('The base URL from which source is retrieved must be set!')
  }

  # Ensure the parent directories exist
  # The default paths are described here
  # https://forge.puppet.com/palli/createrepo#usage
  file { $repo_base_path:
    ensure => 'directory',
  }

  file { $repo_base_cache_path:
    ensure => 'directory',
  }

  # Configure a web-server for the repository
  apache::vhost { $mirror_fqdn:
    port    => '80',
    docroot => '/var/yumrepos/',
  }

  # Print the number of repos in the list
  $repo_count = length($repo_list)
  notice( "Repos: ${repo_count}" )

  # Iterate through the list of repos and create each one
  $repo_list.each |$repo_name, $attributes| {
    # Generate the repo structure
    createrepo { $repo_name: }

    # -- Extract the attributes
    # The revision of the file
    $repo_version = $attributes['version']
    # The first part of the file name
    $prefix = $repo_name
    if has_key($attributes, 'source_prefix') {
      $prefix = $attributes['source_prefix']
    }

    # Create the versions subdirectory if it's not there
    file { "create_${repo_name}_versions":
      ensure => 'directory',
      path   => "${repo_base_path}/${repo_name}/versions",
      owner  => 'root',
      group  => 'root',
    }

    # Clean out any old versions that my exist
    $cleanup_command = "/bin/ls | /bin/grep -v ${repo_version} | /bin/xargs /bin/rm -rf"
    exec { "cleanup_${repo_name}":
      command => $cleanup_command,
      cwd     => "${repo_base_path}/${repo_name}/versions",
      require => File["create_${repo_name}_versions"],
    }

    # Create a subdirectory for the new version
    $repo_rpm_path = "${repo_base_path}/${repo_name}/versions/${repo_version}"
    file { "create_${repo_name}":
      ensure  => 'directory',
      path    => $repo_rpm_path,
      owner   => 'root',
      group   => 'root',
      require => Exec["cleanup_${repo_name}"],
    }

    # Retrieve the latest fileset
    $source_file =
      "${source_base_url}/${prefix}-${repo_version}.tar"
    $dest_file = "/tmp/${prefix}-${repo_version}.tar"
    $retrieve_command =
      "/usr/bin/wget ${source_file} -O ${dest_file}"
    exec { "wget_${repo_name}":
      command => $retrieve_command,
      onlyif  =>
        "/bin/test -n \"\$(/bin/find ${repo_rpm_path} -maxdepth 0 -type d -empty 2>/dev/null)\"",
      creates => "/tmp/${prefix}-${repo_version}.tar",
      require => File["create_${repo_name}"],
    }

    # Update the repository directory
    exec { "extract_${repo_name}":
      command => "/bin/tar -xvf /tmp/${prefix}-${repo_version}.tar",
      cwd     => $repo_rpm_path,
      onlyif  =>
        "/bin/test -n \"\$(/bin/find ${repo_rpm_path} -maxdepth 0 -type d -empty 2>/dev/null)\"",
      require => [
        Exec["wget_${repo_name}"],
        File["create_${repo_name}"],
      ],
    }

    # Correct the ownership on all the resulting files
    $fix_repo_owner =
      "/bin/chown -R root:root ${repo_rpm_path}"
    exec { "fix_owner_${repo_name}":
      command => $fix_repo_owner,
      require => Exec["extract_${repo_name}"],
    }

    # Delete the temporary tar file
    file { "/tmp/${prefix}-${repo_version}.tar":
      ensure  => 'absent',
      require => Exec["fix_owner_${repo_name}"],
    }
  }
}
